/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Indexed Database.
 *
 * The Initial Developer of the Original Code is
 * The Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Turner <bent.mozilla@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIIDBIndex;
interface nsIIDBKeyRange;
interface nsIIDBRequest;
interface nsIIDBTransaction;
interface nsIVariant;
interface nsIDOMDOMStringList;

%{C++
#include "jsapi.h"
%}

/**
 * nsIIDBObjectStore interface.  See
 * http://dev.w3.org/2006/webapi/WebSimpleDB/#idl-def-nsIIDBObjectStore
 * for more information.
 */
[scriptable, uuid(6a65dc92-66e3-407a-a370-590a6c54664a)]
interface nsIIDBObjectStore : nsISupports
{
  readonly attribute DOMString name;

  readonly attribute DOMString keyPath;

  readonly attribute nsIDOMDOMStringList indexNames;

  readonly attribute nsIIDBTransaction transaction;

  // Success fires IDBTransactionEvent, result == value for key
  nsIIDBRequest
  get(in nsIVariant key);

  // Success fires IDBTransactionEvent, result == array of values for given keys
  [optional_argc]
  nsIIDBRequest
  getAll([optional /* null */] in nsIIDBKeyRange key,
         [optional /* unlimited */] in unsigned long limit);

  // Success fires IDBTransactionEvent, result == key
  [implicit_jscontext, optional_argc]
  nsIIDBRequest
  add(in jsval value,
      [optional /* undefined */] in jsval key);

  // Success fires IDBTransactionEvent, result == key
  [implicit_jscontext, optional_argc]
  nsIIDBRequest
  put(in jsval value,
      [optional /* undefined */] in jsval key);

  // Success fires IDBTransactionEvent, result == null
  [implicit_jscontext]
  nsIIDBRequest
  delete(in jsval key);

  // Success fires IDBTransactionEvent, result == null
  nsIIDBRequest
  clear();

  // Success fires IDBTransactionEvent, result == IDBCursor or result == null if
  // no match.
  [optional_argc]
  nsIIDBRequest
  openCursor([optional /* null */] in nsIIDBKeyRange range,
             [optional /* NEXT */] in unsigned short direction);

  /**
   * Optional arguments:
   *   - unique (boolean):
   *       Specifies whether values in the index must be unique. Defaults to
   *       false.
   */
  [implicit_jscontext]
  nsIIDBIndex
  createIndex(in AString name,
              in AString keyPath,
              [optional /* none */] in jsval options);

  // Returns object immediately
  nsIIDBIndex
  index(in AString name);

  void
  deleteIndex(in AString name);
};
